/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.global;

import java.sql.Connection;
import java.sql.Driver;
import java.util.Properties;

import mir.config.MirPropertiesConfiguration;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverConnectionFactory;
import org.apache.commons.dbcp.PoolableConnection;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.GenericObjectPool;

public class DatabaseEngine {
  private ObjectPool connectionPool;

  public DatabaseEngine() {
    MirPropertiesConfiguration configuration = MirPropertiesConfiguration.instance();

    Properties connectionProperties = new Properties();
    connectionProperties.put("user", configuration.getString("Database.Username"));
    connectionProperties.put("password", configuration.getString("Database.Password"));
    String dbHost = configuration.getString("Database.Host");
    String dbPort = configuration.getString("Database.Port");
    String dbName = configuration.getString("Database.Name");
    String dbUrl = "jdbc:postgresql://"+dbHost+":"+dbPort+"/"+dbName;
    String dbDriver = configuration.getString("Database.Driver");
    Driver driver;
    try {
      driver = (Driver) (Class.forName(dbDriver)).newInstance();
    }
    catch (ClassNotFoundException e) {
      throw new Error("JDBC Driver not found: " + e.toString());
    }
    catch (InstantiationException e) {
      throw new Error("Can't instantiate JDBC Driver: " + e.toString());
    }
    catch (IllegalAccessException e) {
      throw new Error("No access to JDBC Driver: " + e.toString());
    }

    connectionPool = new GenericObjectPool(null);
    ConnectionFactory rawConnectionFactory =
        new DriverConnectionFactory(driver, dbUrl, connectionProperties);
    new PoolableConnectionFactory(rawConnectionFactory, connectionPool,
        null, "select now()", false, true);

    // test the connection right before someone borrows it
    ((GenericObjectPool) connectionPool).setTestOnBorrow(true);


  }

  /**
   * Obtain a database connection
   */
  public Connection obtainConnection() throws Exception {
    return (Connection) connectionPool.borrowObject();
  }

  /**
   * release a previously obtained database connection
   */
  public void releaseConnection(Connection aConnection) throws Exception {
    connectionPool.returnObject(aConnection);
  }

  /**
   * Returns the native connection of a previously obtained connection
   */
  public Connection getNativeConnection(Connection aConnection) {
    return ((PoolableConnection) aConnection).getDelegate();
  }

  public String getStatus() {
    return
        connectionPool.getNumActive() + " / " +
        (connectionPool.getNumIdle()+connectionPool.getNumActive()) +
        " db conn.";
  }
}
